﻿#!python3.9
# phy_control_adi.py


import sys, os
import platform
import time, datetime
import json
import operator

from tkinter import filedialog
from tkinter import *
from tkinter import ttk

from broadway2 import Broadway, FCAdapter, RegTracker, RegisterSet, RegisterSetDiff
from broadway2 import get_phy_access, SmiAccess

# uses python phy-modules from local project
# Using FC401 also enables access to ext. devices on EVB/Customer-Boards incl. ADIN1100

from adi import PhyADIN1100

FontSmall = ('Arial', 8)
FontNormal = ('Arial', 10)
FontLarge = ('Arial', 12)
FontHuge = ('Arial', 18)

AppVersion = '2.3.2-1'

class App:
    
    def __init__(self, base, width=0, height=0):
        self.controlPhy = False
        self.base = base
        self.dev = None
        self.currentAdapter = None
        self.togglePhyEnable = False
        self.phy = None
        self.regTracker = RegTracker(10)

        self.root = Tk()
        #self.func = gui_func.guiFunc(self.access,self.root,self.test,self.json)

        #Basic Window Geometry

        self.root.geometry
        self.root.grid(widthInc=width,heightInc=height)

        #Root-Window Apperace

        self.root.configure(background='#44474c')
        self.root.title('PHY Control ADI. V ' + AppVersion)
        self.root.resizable(width=FALSE, height=FALSE)

        #Call Function to Create the Slave Widgets
        self.dlgOpen = False
        self.createWindow()

        #Event Bind

        self.root.bind('<Destroy>',self.shutdown)

        #Mainloop
        self.update_clock()     #initialize timer for GUI cycles
        self.root.mainloop()

    def update_clock(self):
        now = time.strftime("%H:%M:%S")
        #self.label.configure(text=now)
        if self.dev != None:
            if self.togglePhyEnable:
                self.controlPhy = not self.controlPhy
                self.phyEnable.set(self.controlPhy)
                self.phy_enable_changed()
        self.root.after(500, self.update_clock)

    def get_adapter_name(self, adapter):
        if adapter.FriendlyName == '':
            try:
                productName = adapter.DevName
            except:
                productName = 'FC???'
            return '{0}-{1}'.format(productName, adapter.SerialNoBase)
        else:
            return '{0}-{1}-{2}'.format(adapter.DevName, adapter.FriendlyName, adapter.SerialNoBase)

    def get_adapter_list(self):
        """return list of all connected adapters
        in case of FC401, returns multiple entries for each SMI found
        """
        adapters = ['NOT Connected']
        default = ''
        for adapter in self.base.enumerate():
            if default == '':
                default = self.get_adapter_name(adapter)

            if adapter.DeviceType == 401:
                dev = self.base.open_control(adapter.Inst)
                
                smiAccess = SmiAccess(dev)
                phys = smiAccess.detect()

                for phy in phys:
                    if phy[1] == 0x283: # only ADI here
                        adapterName = self.get_adapter_name(adapter) + '#{0}'.format(phy[0])
                        adapters.append(adapterName)

                self.base.close(dev)
            elif adapter.DeviceType == 621 or adapter.DeviceType == 631:
                adapterName = self.get_adapter_name(adapter) + '#0' #builtin_function_or_method-in TJA110x are always on 0
                adapters.append(adapterName)

        return adapters

    def update_reg_window(self):

        self.regVars = {}

        #for widget in self.regBaseFrame.winfo_children():
        #    if isinstance(widget, Label):
        #        widget..destroy()

        frames = {}

        self.nbDev = ttk.Notebook(self.regBaseFrame)

        self.nbDev.grid(column=0, row=0, sticky=(E,W))
        self.nbDev.bind('<<NotebookTabChanged>>',self.nbdev_changed)

        devs = self.phy.get_reg_devs()

        rowIndex = 0
        for dev in devs:
            frames[dev] = Frame(self.regBaseFrame)
            self.nbDev.add(frames[dev], text='Dev-{0:02X}'.format(dev))

            lbregCaption = Label(frames[dev], font = FontNormal, text = 'Register')
            lbregCaption.grid(column=0, row=1, sticky=W)

            lbregCaption = Label(frames[dev], font = FontNormal, text = 'Register Name:')
            lbregCaption.grid(column=1, row=1, sticky=W)

            lbregCaption = Label(frames[dev], font = FontNormal, text = 'Value (Hex):')
            lbregCaption.grid(column=2, row=1, sticky=W)

        rowIndex = 2
        for regdic in self.phy.get_register_sorted():
            devreg = regdic[0]
            dev = (devreg >> 16) & 0xFFFF
            reg = devreg & 0xFFFF

            lbregIdx = Label(frames[dev], font = FontNormal, text = '{0:02X}.{1:04X}:'.format(dev, reg))
            lbregIdx.grid(column=0, row=rowIndex, sticky=W)

            lbreg = Label(frames[dev], font = FontNormal, text = regdic[1]['Name'] + ':')
            lbreg.bind("<Enter>", self.lbreg_enter, regdic[1]['Name'])
            lbreg.bind("<Leave>", self.lbreg_leave)
            lbreg.bind("<Button-3>", self.popup_menu)
            lbreg.grid(column=1, row=rowIndex, sticky=W)

            # create related string variable and store it in reg-index based dic
            regVar = StringVar()
            self.regVars[devreg] = regVar

            lbregVar = Label(frames[dev], font = FontNormal, textvariable = regVar)
            lbregVar.bind("<Enter>", self.lbreg_enter)
            lbregVar.bind("<Leave>", self.lbreg_leave)
            lbregVar.bind("<Button-3>", self.popup_menu)
            lbregVar.grid(column=2, row=rowIndex, sticky=E)

            rowIndex += 1


        self.regDetails = StringVar()
        lbregDetails = Label(self.regBaseFrame, font = FontNormal, textvariable=self.regDetails)
        lbregDetails.grid(column=0, row=rowIndex, columnspan=4, sticky=W)


    def popup_menu(self, event):
        
        try:
            self.regPopup.tk_popup(event.x_root, event.y_root)
        finally:
            self.regPopup.grab_release()

    def createWindow(self):
        """
        ----------------------------------------------------------------
         createWindow(self)

          Function: Build the Slave Widgets of the Root  - Window
        -----------------------------------------------------------------
        """
       
        #Adapter - Label

        self.topFrame = Frame(self.root)
        self.topFrame.grid(column=0,row=0,sticky=W)

        self.topFrame.grid_columnconfigure(2, weight=1, uniform="-------------------")
        self.topFrame.grid_columnconfigure(3, weight=1, uniform="--------------")

        lb = Label(self.topFrame, font = FontLarge, text = 'Adapter:   ')
        lb.configure(background='#FCFCFC')
        lb.grid(column=0,row=0,sticky=W)
        
        #Adapter - Combobox

        adapters = self.get_adapter_list()
        
        self.box = ttk.Combobox(self.topFrame, font = FontLarge, width=52, values = adapters, state = 'readonly')
        self.box.bind('<<ComboboxSelected>>', self.select_adapter)
        self.box.grid(column=1, columnspan=3, row=0, sticky=(W,E))

        currRow = 1

        lb = Label(self.topFrame, font = FontLarge, text = 'Phy-Details:')
        lb.configure(background='#FCFCFC')
        lb.grid(column=0,row=currRow,sticky=W)

         # --- Phy-Type
        currRow += 1

        lbPhyType = Label(self.topFrame, font = FontNormal, text = 'Type:')
        lbPhyType.grid(column=1, row=currRow, sticky=W)

        self.phyType = StringVar()
        self.lbphyType = Label(self.topFrame, font = FontNormal, textvariable=self.phyType)
        self.lbphyType.grid(column=2, row=currRow, sticky=W)

        # --- AN Complete
        currRow += 1

        lbanComplete = Label(self.topFrame, font = FontNormal, text = 'AN Complete:')
        lbanComplete.grid(column=1, row=currRow, sticky=W)

        self.anComplete = StringVar()
        self.lbanComplete = Label(self.topFrame, font = FontNormal, textvariable=self.anComplete)
        self.lbanComplete.grid(column=2, row=currRow, sticky=W)

        # AN link Good
        currRow += 1

        lbanLinkGood = Label(self.topFrame, font = FontNormal, text = 'AN Link Good:')
        lbanLinkGood.grid(column=1, row=currRow, sticky=W)

        self.anLinkGood = StringVar()
        self.lbLinkGood = Label(self.topFrame, font = FontNormal, textvariable=self.anLinkGood)
        self.lbLinkGood.grid(column=2, row=currRow, sticky=W)

        # AN MST Resolution
        currRow += 1

        lbanMstResolution = Label(self.topFrame, font = FontNormal, text = 'AN MST Resolution:')
        lbanMstResolution.grid(column=1, row=currRow, sticky=W)

        self.anMstConfigResolution = StringVar()
        self.lbanMstResolution = Label(self.topFrame, font = FontNormal, textvariable=self.anMstConfigResolution)
        self.lbanMstResolution.grid(column=2, row=currRow, sticky=W)

        # B10L Link-Status
        currRow += 1

        lbb10LLinkStatus = Label(self.topFrame, font = FontNormal, text = 'Link State:')
        lbb10LLinkStatus.grid(column=1, row=currRow, sticky=W)

        self.b10LLinkStatus = StringVar()
        self.lbb10LLinkStatus = Label(self.topFrame, font = FontNormal, textvariable=self.b10LLinkStatus)
        self.lbb10LLinkStatus.grid(column=2, row=currRow, sticky=W)

        # Speed
        currRow += 1

        lbNetworkSpeed = Label(self.topFrame, font = FontNormal, text = 'Speed:')
        lbNetworkSpeed.grid(column=1, row=currRow, sticky=W)

        self.networkSpeed = StringVar()
        self.lbnetworkSpeed = Label(self.topFrame, font = FontNormal, textvariable=self.networkSpeed)
        self.lbnetworkSpeed.grid(column=2, row=currRow, sticky=W)


        #Notebook Widget

        self.nb = ttk.Notebook(self.root)

        # Notebook Phy-Control/Status

        self.phyBaseFrame = Frame(self.root)
        self.phyBaseFrame.grid(sticky=(N,W))

        currRow = 1
        lbregEnable = Label(self.phyBaseFrame, font = FontNormal, text = 'Tx Mode Disable')
        lbregEnable.grid(column=0, row=currRow, sticky=W)

        self.b10TxDisable = BooleanVar()

        self.cbb10TxDisable = Checkbutton(self.phyBaseFrame,variable=self.b10TxDisable, command=self.b10_tx_disable_changed)
        self.cbb10TxDisable.grid(column=1,row=currRow,padx=4,pady=2,sticky=W)
        self.cbb10TxDisable.config(state = DISABLED)

        currRow += 1

        lbenV24 = Label(self.phyBaseFrame, font = FontNormal, text = 'Enable 2.4V')
        lbenV24.grid(column=0, row=currRow, sticky=W)

        self.enV24 = BooleanVar()

        self.cbV24 = Checkbutton(self.phyBaseFrame,variable=self.enV24, command=self.en_v24_changed)
        self.cbV24.grid(column=1,row=currRow,padx=4,pady=2,sticky=W)
        self.cbV24.config(state = DISABLED)

        currRow += 1

        lbanAdvMaster = Label(self.phyBaseFrame, font = FontNormal, text = 'Adv. Master')
        lbanAdvMaster.grid(column=0, row=currRow, sticky=W)

        self.anAdvMaster = BooleanVar()

        self.cbAdvMaster = Checkbutton(self.phyBaseFrame,variable=self.anAdvMaster, command=self.an_adv_master_changed)
        self.cbAdvMaster.grid(column=1,row=currRow,padx=4,pady=2,sticky=W)
        self.cbAdvMaster.config(state = DISABLED)

        currRow += 1

        lbanForceMaster = Label(self.phyBaseFrame, font = FontNormal, text = 'Force Master')
        lbanForceMaster.grid(column=0, row=currRow, sticky=W)

        self.anForcevMaster = BooleanVar()

        self.cbForceMaster = Checkbutton(self.phyBaseFrame,variable=self.anForcevMaster, command=self.an_force_master_changed)
        self.cbForceMaster.grid(column=1,row=currRow,padx=4,pady=2,sticky=W)
        self.cbForceMaster.config(state = DISABLED)


        self.phyAutoOpMode = BooleanVar()

        self.cbAutoOpMode = Checkbutton(self.phyBaseFrame,variable=self.phyAutoOpMode, command=self.phy_auto_op_mode_changed)
        self.cbAutoOpMode.grid(column=2,row=currRow,padx=4,pady=2,sticky=W)
        self.cbAutoOpMode.config(state = DISABLED)

        lbAutoOpMode = Label(self.phyBaseFrame, font = FontNormal, text = 'AutoOp')
        lbAutoOpMode.grid(column=3, row=currRow, sticky=W)

        
        # --- Loopback
        currRow += 1

        lbLoopMode = Label(self.phyBaseFrame, font = FontNormal, text = 'Loopback:')
        lbLoopMode.grid(column=0, row=currRow, sticky=W)
        
        LoopModes = ['0 - Loopback OFF']
        LoopModes.append('1 - PCS Loopback')
        LoopModes.append('2 - MAC Loopback')
        LoopModes.append('3 - MAC Remote-Loopback')
       
        self.boxLoopModes = ttk.Combobox(self.phyBaseFrame, font = FontNormal, values = LoopModes, width=25, state = 'readonly')
        self.boxLoopModes.bind('<<ComboboxSelected>>', self.select_loopBack_mode)
        self.boxLoopModes.grid(column=1, row=currRow, sticky=W)

        # --- Test-Modes
        currRow += 1

        lbTestMode = Label(self.phyBaseFrame, font = FontNormal, text = 'TestMode:')
        lbTestMode.grid(column=0, row=currRow, sticky=W)
        
        TestModes = ['0 - TestMode OFF']
        TestModes.append('1 - TestMode 1')
        TestModes.append('2 - TestMode 2')
        TestModes.append('3 - TestMode 3')
       
        self.boxTestModes = ttk.Combobox(self.phyBaseFrame, font = FontNormal, values = TestModes, width=25, state = 'readonly')
        self.boxTestModes.bind('<<ComboboxSelected>>', self.select_test_mode)
        self.boxTestModes.grid(column=1, row=currRow, sticky=W)

        #Notebook Register Overview Tab

        self.regBaseFrame = Frame(self.root)
        self.regBaseFrame.grid(column=0, row=1, sticky=(N,W))

        self.regPopup = Menu(self.regBaseFrame,tearoff=0)        ## Popup-Menue

        self.pollRegs = BooleanVar()
        self.regPopup.add_checkbutton(label='Polling active', variable=self.pollRegs, command=self.chkPoll_Changed)
        self.regPopup.add_separator()
        self.regPopup.add_command(label='Copy to Clipboard', command=self.btn_reg_copy_clicked)
        self.regPopup.add_command(label='Save to file ...', command=self.btn_saveregs_clicked)

        # Notebook Info Tab

        self.infoFrame = Frame(self.root)
        self.infoFrame.grid(sticky=(N,W))

        currRow = 2

        lbCompany = Label(self.infoFrame, font = FontNormal, text = 'Company:')
        lbCompany.grid(column=0, row=currRow, sticky=W)

        lbCompanyFC = Label(self.infoFrame, font = FontNormal, text = 'FibreCode GmbH')
        lbCompanyFC.grid(column=1, row=currRow, sticky=W)

        currRow += 1

        lbSerialBase = Label(self.infoFrame, font = FontNormal, text = 'SerialNo Base:')
        lbSerialBase.grid(column=0, row=currRow, sticky=W)

        self.serialNumberBase = StringVar()
        self.lbSerialBaseA = Label(self.infoFrame, font = FontNormal, textvariable=self.serialNumberBase)
        self.lbSerialBaseA.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # SerialNo AddOn

        lbSerialAddOnTitle = Label(self.infoFrame, font = FontNormal, text = 'SerialNo AddOn:')
        lbSerialAddOnTitle.grid(column=0, row=currRow, sticky=W)

        self.serialNumberAddOn = StringVar()
        self.lbSerialAddOnA = Label(self.infoFrame, font = FontNormal, textvariable=self.serialNumberAddOn)
        self.lbSerialAddOnA.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # FW Version

        lblFWVersion = Label(self.infoFrame, font = FontNormal, text = 'FW Version:')
        lblFWVersion.grid(column=0, row=currRow, sticky=W)

        self.fwVersion = StringVar()
        self.lbFWVersion = Label(self.infoFrame, font = FontNormal, textvariable=self.fwVersion)
        self.lbFWVersion.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # ID

        lbid = Label(self.infoFrame, font = FontNormal, text = 'ID:')
        lbid.grid(column=0, row=currRow, sticky=W)

        self.id = StringVar()
        self.lbid = Label(self.infoFrame, font = FontNormal, textvariable=self.id)
        self.lbid.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # MAC-Address

        lbmac = Label(self.infoFrame, font = FontNormal, text = 'MAC-Address:')
        lbmac.grid(column=0, row=currRow, sticky=W)

        self.macAddr = StringVar()
        self.lbMacAddr = Label(self.infoFrame, font = FontNormal, textvariable=self.macAddr)
        self.lbMacAddr.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # Friendly Name

        lbmac = Label(self.infoFrame, font = FontNormal, text = 'FriendlyName:')
        lbmac.grid(column=0, row=currRow, sticky=W)

        self.friendlyName = StringVar()
        self.lbfriendlyName = Label(self.infoFrame, font = FontNormal, textvariable=self.friendlyName)
        self.lbfriendlyName.grid(column=1, row=currRow, sticky=W)

        currRow += 1    # Stick Time

        lbstickTime = Label(self.infoFrame, font = FontNormal, text = 'StickTime:')
        lbstickTime.grid(column=0, row=currRow, sticky=W)

        self.stickTime = StringVar()
        self.lbstickTime = Label(self.infoFrame, font = FontNormal, textvariable=self.stickTime)
        self.lbstickTime.grid(column=1, row=currRow, sticky=W)

        #Add Tabs to Notebook
        self.nb.add(self.phyBaseFrame, text='Phy-Control/Status')
        
        self.nb.add(self.regBaseFrame, text='SMI-Register')
        self.nb.add(self.infoFrame, text='Info')
        
        #Grid Notebook

        self.nb.grid(column=0, sticky=(E,W))
        self.nb.bind('<<NotebookTabChanged>>',self.nb_changed)

# EVENT Handler
    def shutdown(self, event):
        """close all open instances to clean-up"""
        if self.dlgOpen:
            return

        if self.dev != None:
            self.phy.disconnect()
            self.base.close(self.dev)
            self.dev = None

    def select_adapter(self, event):
        """ select_adapter(self, event)

          event = Instance of a VirtualEvent.
        """

        if self.dev != None:
            self.phy.disconnect()
            self.base.close(self.dev)
            self.dev = None

        for adapter in self.base.enumerate():
            # adapterName#phyAddr as list in combo
            select  = self.box.get().split('#')
            if self.get_adapter_name(adapter) == select[0]:
                # register update
                self.dev = self.base.open_control(adapter.Inst)
                
                self.cbb10TxDisable.config(state = ACTIVE)
                self.cbAdvMaster.config(state = ACTIVE)
                self.cbForceMaster.config(state = ACTIVE)
                self.cbAutoOpMode.config(state = ACTIVE)
                self.cbV24.config(state = ACTIVE)
                #self.cbLinkControl.after(200, self.__update_gui)
                self.currentAdapter = adapter
                self.currentAdapter.connect(self.dev)

                phyAccess = get_phy_access(adapter, int(select[1]))

                phyid = (phyAccess.read_dev_reg(0x1E, 2) << 16) | phyAccess.read_dev_reg(0x1E, 3)

                # verify if phyid is supported. If yes, create helper-class to have registers translated to properties/methods
                if phyid == 0x283BC80 or phyid == 0x283BC81:      # ADIN1100 MRA1 or MRA2
                    self.phy = PhyADIN1100(phyAccess)
                    self.phyType.set('ADIN1100')
                else:
                    print('UNSUPPORTED PHY!!!!')
                    return
               
                self.update_reg_window()
                time.sleep(0.1)
                self.pollRegs.set(True)
                self.chkPoll_Changed()
                self.__update_gui(True)

    def __update_gui(self, initialize=False):
        adapter = self.currentAdapter

        for devreg in self.phy.get_register():
            self.regVars[devreg].set('{0:4X}'.format(self.phy.read_register(devreg)))

        if initialize:
            self.b10TxDisable.set(self.phy.b10TxOutputDisable)
            self.anAdvMaster.set(self.phy.AnAdvMaster)
            self.anForcevMaster.set(self.phy.AnAdvForceMaster)
            self.enV24.set(self.phy.EnableV24)

            loopbackMode = self.phy.get_b10l_loopback_mode()
            self.boxLoopModes.current(loopbackMode)
            testMode = self.phy.get_b10l_test_mode()
            self.boxTestModes.current(testMode)

        # info tab
        self.serialNumberBase.set(adapter.SerialNoBase)
        self.serialNumberAddOn.set(adapter.SerialNoAddOn)
        self.fwVersion.set(adapter.FWRevision)
        self.id.set(adapter.SiliconKey)
        self.macAddr.set(adapter.MACAddress)
        self.friendlyName.set(adapter.FriendlyName)

        self.stickTime.set(adapter.get_timestamp())

       
        # status tab
        self.anComplete.set(self.phy.AnComplete)
        self.anLinkGood.set(self.phy.AnLinkGood)
        self.anMstConfigResolution.set(self.phy.AnMSConfigResolution)
        self.b10LLinkStatus.set(self.phy.B10LLinkStatus)
        self.networkSpeed.set(self.phy.NetworkSpeed)
        
    def nb_changed(self,event):
        """
        ----------------------------------------------------------------
         nb_changed(self,event)

          event = Event-Class of the bind Event

          Function: Function called if the selected Tab changes. Function
                    configures the Apperence of several Items in the GUI.
        -----------------------------------------------------------------
        """
        #Button in the Register Overview Window
        index = self.nb.index(self.nb.select())

        if self.dev == None:
            return

        self.__update_gui()

    def nbdev_changed(self,event):
        """
        ----------------------------------------------------------------
         nb_changed(self,event)

          event = Event-Class of the bind Event

          Function: Function called if the selected Tab changes. Function
                    configures the Apperence of several Items in the GUI.
        -----------------------------------------------------------------
        """
        #Button in the Register Overview Window
        index = self.nbDev.index(self.nbDev.select())

        if self.dev == None:
            return

        self.__update_gui()


    def lbreg_enter(self, event):
        if self.dev == None:
            return
        register = self.phy.get_register()
        for reg in register:
            if register[reg]['Name'] + ':' == event.widget.cget('text'):
                regValue = self.phy.read_register(reg)
                regDetails = self.phy.decode(reg, regValue)
                print('Register: {0:02X}.{1:04X} = 0x{2:04X}'.format(reg>>16, reg&0xFFFF, regValue))
                print('-'*48)
                txtOut = ''
                for bits in regDetails:
                    if bits[1] > 1:
                        txtOut += '{0:02}:{1:02} {2} = {3} - {4}\n'.format(bits[0]+bits[1]-1, bits[0], bits[2], bits[4], bits[3])
                    else:
                        txtOut += '{0:02}:{1} {2} = {3} - {4}\n'.format(bits[0], '  ', bits[2], bits[4], bits[3])
                print(txtOut)
                self.regDetails.set('Selected: {0}'.format(register[reg]['Name']))
                break

    def lbreg_leave(self, event):
        self.regDetails.set('')

    def config_enable_changed(self):
        self.phy.configEnable = self.configEnable.get()
        time.sleep(0.1)
        self.__update_gui()

    def en_v24_changed(self):
        self.phy.EnableV24 = self.enV24.get()
        time.sleep(0.1)
        self.__update_gui()

    def b10_tx_disable_changed(self):
        self.phy.b10TxOutputDisable = self.b10TxDisable.get()
        time.sleep(0.1)
        self.__update_gui()
        
    def an_adv_master_changed(self):
        self.phy.AnAdvMaster = self.anAdvMaster.get()
        time.sleep(0.1)
        self.__update_gui()

    def an_force_master_changed(self):
        self.phy.AnAdvForceMaster = self.anForcevMaster.get()
        time.sleep(0.1)
        self.__update_gui()

    def phy_auto_op_mode_changed(self):
        self.phy.AutoMode = self.phyAutoOpMode.get()
        time.sleep(0.1)
        self.__update_gui()


    def select_loopBack_mode(self, event):
        loopModeStr = self.boxLoopModes.get()
        try:
            loopbackMode = int(loopModeStr[0])
        except:
            loopbackMode = 0

        if loopbackMode == 0:
            self.phy.disable_loopback_mode()
        else:
            self.phy.enable_loopback_mode(loopbackMode)

    def select_test_mode(self, event):
        testModeStr = self.boxTestModes.get()
        try:
            testMode = int(testModeStr[0])
        except:
            testMode = 0

        if testMode == 0:
            self.phy.disable_b10l_test_mode()
        else:
            self.phy.enable_b10l_test_mode(testMode)

    def btn_reg_copy_clicked(self):
        jsonOut = {}
        jsonOut['Class'] = 'RegisterDump'
        jsonOut['DateTime'] = str(datetime.datetime.now())
        jsonOut['Comment'] = ''
        jsonOut['Register'] = []
        jsonOut['RegisterDetails'] = []
        for reg in self.phy.get_register():
            regValue = self.phy.read_register(reg)
            regDetails = self.phy.decode(reg, regValue)
            jsonOut['Register'].append({'Reg': reg, 'Tag': self.phy.lookup_register(reg), 'Value': regValue})
            jsonOut['RegisterDetails'].append({'Reg': reg, 'Value': regValue, 'Details': regDetails})
        clpStr = json.dumps(jsonOut, indent=4)
        self.root.clipboard_clear()
        self.root.clipboard_append(clpStr)

    def chkPoll_Changed(self):
        # register change notifications (display only)
        # added exceptions for Reg21, Reg24 and Reg25 (bits are toggling fast)
        if self.pollRegs.get() and self.phy.isConnected:
            
            rs = RegisterSet()
            rs.trackStart = datetime.datetime.now()
            for devreg in self.phy.get_register():
                
                rs.regValues[devreg] = self.phy.read_register(devreg)

                
            rs.trackEnd = datetime.datetime.now()
            res = self.regTracker.add_track_if_changed(rs)
            if res:
                for reg in self.phy.get_register():
                    self.regVars[reg].set('{0:4X}'.format(rs.regValues[reg]))
                regDiff = self.regTracker.get_last_diff()

                print('Register changes: {0} {1} {2}'.format(regDiff.numDifference, regDiff.trackStart, regDiff.trackDiffTime))
                for cregs in regDiff.regDiffValues:
                    print('{0:02X}.{1:04X} : {2:04X}({3:04X})'.format(cregs>>16, cregs & 0xFFFF, regDiff.regDiffValues[cregs][0], regDiff.regDiffValues[cregs][1]))

            #print('INH={0}'.format(self.currentAdapter.query_inh()))
            self.__update_gui()
        self.root.after(100, self.chkPoll_Changed)

    def select_comment(self):
        pass

    def btn_saveregs_clicked(self):

        tmName = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')

        self.dlgOpen = True
        fname = filedialog.asksaveasfilename(defaultextension=".txt", initialfile='{0}_{1}'.format(self.phyType.get(), tmName), title = "Select file",filetypes = (("text files","*.txt"),("ini files","*.ini")))
        self.dlgOpen = False
        if fname == '':
            return

        f = open(fname, mode="w", encoding='UTF-8')

        tmStart = datetime.datetime.now()
        regValues = {}
        for reg in self.phy.get_register():
            regValues[reg] = self.phy.read_register(reg)

        tmEnd = datetime.datetime.now()

        f.write('REGISTER DUMP: {0}\n'.format(self.phyType.get()))
        f.write('Start: {0}\n'.format(str(tmStart)))
        f.write('Stop:  {0}\n'.format(str(tmEnd)))
        for reg in sorted(self.phy.get_register()):
            f.write('{0:02X}.{1:04X} = 0x{2:04X}\n'.format(reg>>16, reg & 0xFFFF, regValues[reg]))

        f.write('{0}\n'.format(48*'-'))

        for reg in sorted(self.phy.get_register()):
            regTag = self.phy.lookup_register(reg)
            regValue = regValues[reg]
            regDetails = self.phy.decode(reg, regValue)
            f.write('Reg: {0:02X}.{1:04X} ({2}) = 0x{3:04X}\n'.format(reg>>16, reg & 0xFFFF, regTag, regValue))
            for bits in regDetails:
                try:
                    if bits[1] > 1:
                        txtOut = '{0:02}:{1:02} {2} = {3} - {4}\n'.format(bits[0]+bits[1]-1, bits[0], bits[2], bits[4], bits[3])
                    else:
                        txtOut = '{0:02}:{1} {2} = {3} - {4}\n'.format(bits[0], '  ', bits[2], bits[4], bits[3])
                    f.write('\t{0}'.format(txtOut))
                except:
                    f.write('\t{0}'.format(bits))

        f.close()



def main(args):
     # create adapter singleton
    base = Broadway()

    # will never return until gui exits
    app = App(base, 100, 50)

if __name__ == "__main__":
    main(sys.argv)
